#ifndef TILES_SRAM_BUFFER_H
#define TILES_SRAM_BUFFER_H

#include "SpriteSet.h"
#include "TileSet.h"
#include "TileMap.h"

byte *sram[BUFFER_W];

void Copy(byte *src, byte *dst)
{
  __asm__ __volatile(
    "lpm r0,z+\n\t"
    "st  y+,r0\n\t"
    "lpm r0,z+\n\t"
    "st  y+,r0\n\t"
    "lpm r0,z+\n\t"
    "st  y+,r0\n\t"
    "lpm r0,z+\n\t"
    "st  y+,r0\n\t"
    "lpm r0,z+\n\t"
    "st  y+,r0\n\t"
    "lpm r0,z+\n\t"
    "st  y+,r0\n\t"
    "lpm r0,z+\n\t"
    "st  y+,r0\n\t"
    "lpm r0,z+\n\t"
    "st  y+,r0\n\t"
    :
    : "z" (src), "y" (dst)
    :
  );
}

void Copy2(byte *src, byte *dst)
{
  byte i;
  byte s;
  for(i=0; i<TILE_SIZE; i++)
  {
    s = pgm_read_byte(&(src[i]));
    if (s!=TransparentColor)
      dst[i] = s;
  }
}

void InitSRAM()
{

  // Enable MegaRAM shield external memory
  XMCRA = _BV(SRE);

  // Disable high memory masking (don't need >56kbytes in this app)
  XMCRB = 0;

  // Enable RAM device
  pinMode(42, OUTPUT); digitalWrite(42, HIGH);

  // Enable bank select output and select bank 0. Don't need dual banks in this app.
  pinMode(38, OUTPUT); digitalWrite(38, LOW);
  
  // Alloc buffer
  for(unsigned int y=0; y<BUFFER_H; y++ )
    sram[y] = (byte *) 0x2200U + (y * BUFFER_W);

}

void SetSprite(byte x, byte y, byte sprite)
{
  y *= TILE_SIZE;
  x *= TILE_SIZE;
  for(byte ty=0; ty<TILE_SIZE; ty++)
    Copy2(SpriteSet[sprite][ty], &sram[y+ty][x]);
}

void SetSprite(byte x, byte y)
{
  byte tile = TileMap[y][x];
  SetSprite(x, y, tile);
}

inline void SetTile(byte x, byte y, byte tile)
{
  y *= TILE_SIZE;
  x *= TILE_SIZE;
  for(byte ty=0; ty<TILE_SIZE; ty++)
    Copy(TileSet[tile][ty], &sram[y+ty][x]);
}

void SetTile(byte x, byte y)
{
  byte tile = TileMap[y][x];
  SetTile(x, y, tile);
}

void FillSRAM()
{
  for(byte y=0; y<BUFFER_H/8; y++)
    for(byte x=0; x<BUFFER_W/8; x++)
      SetTile(x, y);
}

#endif
