/********************************************************************************************/
/*   ARDUINO SPRITES                                                                        */
/*   RENDERING FROM AN SRAM FRAMEBUFFER                                                     */
/********************************************************************************************/
/* Author: Raul Navarro                                                                     */
/* Web:    http://www.raulnd.com                                                            */
/* E-Mail: raulnd@gmail.com                                                                 */
/********************************************************************************************/
/*                                                                                          */
/* This program is free software: you can redistribute it and/or modify                     */
/* it under the terms of the GNU General Public License as published by                     */
/* the Free Software Foundation, either version 3 of the License, or                        */
/* (at your option) any later version.                                                      */
/*                                                                                          */
/* This program is distributed in the hope that it will be useful,                          */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of                           */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                            */
/* GNU General Public License for more details.                                             */
/*                                                                                          */
/* You should have received a copy of the GNU General Public License                        */
/* along with this program.  If not, see  < http://www.gnu.org/licenses/ >.                 */
/*                                                                                          */
/********************************************************************************************/
/*                                                                                          */
/* Methods used for PWM sync have been extracted from the TVOut library.                    */
/* URL: http://code.google.com/p/arduino-tvout/                                             */
/*                                                                                          */
/* RenderLine and RenderSRAM methods are inspired in the example code of Masami Watanabe:   */
/* Playing with Arduino (File: INT_OutDataL_160.cpp)                                        */
/* URL: http://homepage3.nifty.com/two_legs/neo/project/Arduino/proj_Arduino.htm            */
/*                                                                                          */
/* Shield used:                                                                             */
/* MegaRAM: http://ruggedcircuits.com/html/megaram.html                                     */
/*                                                                                          */
/********************************************************************************************/
/*                                                                                          */
/*   TESTED ON COMODORE 1084S - DIN6 CONNECTOR                                              */
/*     _______                                                                              */
/*    /5  º  1\      1: GREEN      4: RED                                                   */
/*   |4   6   2|     2: H. SYNC    5: BLUE                                                  */
/*    \___3___/      3: GND        6: V. SYNC                                               */
/*                                                                                          */
/*   ARDUINO               DIN6                                                             */
/*   SYNC_PIN ------------- 2                                                               */
/*                                                                                          */
/*   R1_PIN   ---[750R]--+- 4                                                               */
/*   R0_PIN   ---[ 1K5]--+                                                                  */
/*                                                                                          */
/*   G1_PIN   ---[750R]--+- 1                                                               */
/*   G0_PIN   ---[ 1K5]--+                                                                  */
/*                                                                                          */
/*   B1_PIN   ---[750R]--+- 5                                                               */
/*   B0_PIN   ---[ 1K5]--'                                                                  */
/*                                                                                          */
/*   GND      ------------- 3                                                               */
/*   GND      ------------- 6                                                               */
/*                                                                                          */
/********************************************************************************************/

#define PAL

// Compatibilidad con Arduino 1.0 IDE
#if defined(ARDUINO) && ARDUINO >= 100
  #include "Arduino.h"
#else
  #include "WProgram.h"
#endif

#include "Constants.h"
#include "SRAMBuffer.h"
#include "VideoGen.h"

/********************************************************************************************/

void setup()
{

  //pinMode (H_PIN, OUTPUT);
  //pinMode (V_PIN, OUTPUT);
  pinMode (SYNC_PIN, OUTPUT);
  pinMode (R1_PIN, OUTPUT);
  pinMode (R0_PIN, OUTPUT);
  pinMode (G1_PIN, OUTPUT);
  pinMode (G0_PIN, OUTPUT);
  pinMode (B1_PIN, OUTPUT);
  pinMode (B0_PIN, OUTPUT);
  
  //digitalWrite (H_PIN, HIGH);
  //digitalWrite (V_PIN, HIGH);
  digitalWrite (SYNC_PIN, HIGH);
  digitalWrite (R1_PIN, HIGH);
  digitalWrite (R0_PIN, HIGH);
  digitalWrite (G1_PIN, HIGH);
  digitalWrite (G0_PIN, HIGH);
  digitalWrite (B1_PIN, HIGH);
  digitalWrite (B0_PIN, HIGH);

  InitSRAM();
  FillSRAM();

  render_setup();
  
}

byte spr;
byte sprC = 0;
byte sprW = 3;

void loop()
{
// Game update code

spr = sprC * sprW;

// Backbround
SetTile(2,11);
SetTile(3,11);
SetTile(4,11);
SetTile(2,12);
SetTile(3,12);
SetTile(4,12);
SetTile(2,13);
SetTile(3,13);
SetTile(4,13);

// Character
SetSprite(2,11,spr);
SetSprite(3,11,spr+1);
SetSprite(4,11,spr+2);
SetSprite(2,12,spr+12);
SetSprite(3,12,spr+13);
SetSprite(4,12,spr+14);
SetSprite(2,13,spr+24);
SetSprite(3,13,spr+25);
SetSprite(4,13,spr+26);

sprC = (sprC+1)%4;
delay(100);


}

