/* URL: http://code.google.com/p/arduino-tvout/ */

/*
 Copyright (c) 2010 Myles Metzer

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef VIDEO_GEN_H
#define VIDEO_GEN_H

#include "VideoProperties.h"
#include "RenderLine.h"

typedef struct
{
  volatile int             scanLine;
  int                      renderLine;
  volatile unsigned long   frames;
  uint8_t                **screen;
} TVout_vid;

TVout_vid display;

void (*line_handler)();

void blank_line();
void active_line();
void vsync_line();

static void inline wait_until(uint8_t time)
{
  __asm__ __volatile__ (
      "subi  %[time], 10\n"
      "sub  %[time], %[tcnt1l]\n\t"
    "100:\n\t"
      "subi  %[time], 3\n\t"
      "brcc  100b\n\t"
      "subi  %[time], 0-3\n\t"
      "breq  101f\n\t"
      "dec  %[time]\n\t"
      "breq  102f\n\t"
      "rjmp  102f\n"
    "101:\n\t"
      "nop\n" 
    "102:\n"
    :
    : [time] "a" (time),
    [tcnt1l] "a" (TCNT1L)
  );
}

void render_setup()
{

  display.screen = sram;
  display.frames = 0;
  
  // inverted fast pwm mode on timer 1
  TCCR1A = _BV(COM1A1) | _BV(COM1A0) | _BV(WGM11);
  TCCR1B = _BV(WGM13) | _BV(WGM12) | _BV(CS10);

  ICR1  = _CYCLES_SCANLINE;
  OCR1A = _CYCLES_HORZ_SYNC;
  
  display.scanLine = _LINE_FRAME + 1;
  line_handler = &vsync_line;
  TIMSK1 = _BV(TOIE1);
  sei();
  
}

// render a line
ISR(TIMER1_OVF_vect) {
  line_handler();
}

void blank_line()
{
  if ( display.scanLine == _LINE_START_RENDER) // Inicio de RENDER
  {
    display.renderLine = 0;
    line_handler = &active_line;
  }
  else if (display.scanLine == _LINE_FRAME) // Final del frame
  {
    line_handler = &vsync_line;
  }
  display.scanLine++;
}

void active_line()
{

  wait_until(_CYCLES_OUTPUT_START); //Esperar sincronia

  RenderSRAM(display.screen[display.renderLine>>1]);
  //RenderSRAM2(display.screen[display.renderLine>>1]);
  
  PORT = _SYNC;
  display.renderLine++;

  if(display.renderLine == DISPLAY_LINES)
    line_handler = &blank_line;
    
  display.scanLine++;
  
}

void vsync_line()
{
  if (display.scanLine >= _LINE_FRAME) // Final del frame
  {
    OCR1A = _CYCLES_VIRT_SYNC;
    display.scanLine = 0;
    display.frames++;
  }
  else if (display.scanLine == _LINE_STOP_VSYNC) // Fin de VSYNC
  {
    OCR1A = _CYCLES_HORZ_SYNC;
    line_handler = &blank_line;
  }
  display.scanLine++;
}

#endif

